# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from django.conf import settings
from django.contrib.auth import (
    authenticate, login, logout, REDIRECT_FIELD_NAME)
from django.contrib.auth.models import User
from django.core.exceptions import (
    ImproperlyConfigured, ObjectDoesNotExist, PermissionDenied)
from django.urls import reverse_lazy
from onelogin.saml2.auth import OneLogin_Saml2_Auth
from uw_saml.utils import get_user


class DjangoSAML(object):
    """
    This class acts as a wrapper around an instance of either a
    OneLogin_Saml2_Auth or a Mock_Saml2_Auth class.
    """
    FORWARDED_HOST = 'HTTP_X_FORWARDED_HOST'
    FORWARDED_PORT = 'HTTP_X_FORWARDED_PORT'
    FORWARDED_PROTO = 'HTTP_X_FORWARDED_PROTO'

    ATTRIBUTE_MAP = {
        'urn:oid:0.9.2342.19200300.100.1.1': 'uwnetid',
        'urn:oid:1.3.6.1.4.1.5923.1.1.1.6': 'eppn',
        'urn:oid:1.2.840.113994.200.24': 'uwregid',
        'urn:oid:0.9.2342.19200300.100.1.3': 'email',
        'urn:oid:2.16.840.1.113730.3.1.241': 'displayName',
        'urn:oid:2.5.4.42': 'givenName',
        'urn:oid:2.5.4.4': 'surname',
        'urn:oid:1.2.840.113994.200.21': 'studentid',
        'urn:oid:2.16.840.1.113730.3.1.3': 'employeeNumber',
        'urn:oid:2.5.4.11': 'homeDepartment',
        'urn:oid:1.3.6.1.4.1.5923.1.1.1.1': 'affiliations',
        'urn:oid:1.3.6.1.4.1.5923.1.1.1.9': 'scopedAffiliations',
        'urn:oid:1.3.6.1.4.1.5923.1.5.1.1': 'isMemberOf',
    }
    GROUP_NS = 'urn:mace:washington.edu:groups:'

    def __init__(self, request):
        self._request = request

        if hasattr(settings, 'MOCK_SAML_ATTRIBUTES'):
            self._implementation = Mock_Saml2_Auth()
            self.process_response()

        elif hasattr(settings, 'DJANGO_LOGIN_MOCK_SAML'):
            self._implementation = Django_Login_Mock_Saml2_Auth(request)

        elif hasattr(settings, 'UW_SAML'):
            request_data = {
                'https': 'on' if request.is_secure() else 'off',
                'http_host': request.META['HTTP_HOST'],
                'script_name': request.META['PATH_INFO'],
                'server_port': request.META['SERVER_PORT'],
                'get_data': request.GET.copy(),
                'post_data': request.POST.copy(),
                'query_string': request.META['QUERY_STRING']
            }

            if self.FORWARDED_HOST in request.META:
                request_data['http_host'] = request.META[self.FORWARDED_HOST]

            if self.FORWARDED_PORT in request.META:
                request_data['server_port'] = request.META[self.FORWARDED_PORT]

            if self.FORWARDED_PROTO in request.META:
                request_data['https'] = 'on' if (
                    request.META[self.FORWARDED_PROTO] == 'https') else 'off'

            self._implementation = OneLogin_Saml2_Auth(
                request_data, old_settings=getattr(settings, 'UW_SAML'))

        else:
            raise ImproperlyConfigured('Missing "UW_SAML" dict in settings.py')

    def __getattr__(self, name, *args, **kwargs):
        """
        Pass unshimmed method calls through to the implementation instance.
        """
        def handler(*args, **kwargs):
            return getattr(self._implementation, name)(*args, **kwargs)
        return handler

    def login(self, **kwargs):
        """
        Overrides the implementation method to add force_authn option.
        """
        kwargs['force_authn'] = getattr(settings, 'SAML_FORCE_AUTHN', False)
        return self._implementation.login(**kwargs)

    def logout(self, **kwargs):
        """
        Overrides the implementation method to add the Django logout.
        """
        kwargs['name_id'] = self._request.session.get('samlNameId')
        kwargs['session_index'] = self._request.session.get('samlSessionIndex')

        # Django logout
        logout(self._request)

        return self._implementation.logout(**kwargs)

    def process_response(self):
        """
        Overrides the implementation method to store the SAML attributes and
        add the Django login.
        """
        self._implementation.process_response()

        self._request.session['samlUserdata'] = self.get_attributes()
        self._request.session['samlNameId'] = self.get_nameid()
        self._request.session['samlSessionIndex'] = self.get_session_index()

        # Django login
        user = authenticate(self._request, remote_user=get_user(self._request))
        login(self._request, user)

    def get_attributes(self):
        """
        Overrides the implementation method to return a dictionary of SAML
        attributes, mapping the default names to friendlier names.
        """
        attributes = {self.ATTRIBUTE_MAP.get(key, key): val for key, val in (
            self._implementation.get_attributes().items())}

        if 'isMemberOf' in attributes:
            attributes['isMemberOf'] = [e.replace(self.GROUP_NS, '') for e in (
                attributes['isMemberOf'])]

        return attributes


class Mock_Saml2_Auth(object):
    def login(self, **kwargs):
        return kwargs.get('return_to', '')

    def logout(self, **kwargs):
        return kwargs.get('return_to', '')

    def process_response(self):
        return

    def get_attributes(self):
        return getattr(settings, 'MOCK_SAML_ATTRIBUTES')

    def get_nameid(self):
        return 'mock-nameid'

    def get_session_index(self):
        return 'mock-session-index'


class Django_Login_Mock_Saml2_Auth(object):
    def __init__(self, request):
        self.dl_saml_data = getattr(
            settings, 'DJANGO_LOGIN_MOCK_SAML'
        )
        for user in self.dl_saml_data['SAML_USERS']:
            try:
                User.objects.get(username=user["username"])
            except ObjectDoesNotExist:
                User.objects.create_user(
                    user["username"],
                    email=user["email"],
                    password=user["password"]
                ).save()
        self.request = request

    def login(self, **kwargs):
        return "{}?{}={}".format(
            reverse_lazy('login_django'), REDIRECT_FIELD_NAME,
            kwargs.get('return_to', '')
        )

    def logout(self, **kwargs):
        return kwargs.get('return_to', '')

    def process_response(self):
        if self.request.user.is_authenticated:
            self.username = self.request.user.username
        else:
            raise PermissionDenied(
                'The request must be authenticated before it can be processed'
            )
        return

    def get_attributes(self):
        for i, user in enumerate(self.dl_saml_data['SAML_USERS']):
            if (user["username"] == self.username):
                return user['MOCK_ATTRIBUTES']
        raise ImproperlyConfigured('This user does not exist in SAML_USERS')

    def get_nameid(self):
        if 'NAME_ID' in self.dl_saml_data:
            return self.dl_saml_data['NAME_ID']
        return 'mock-nameid'

    def get_session_index(self):
        if 'SESSION_INDEX' in self.dl_saml_data:
            return self.dl_saml_data['SESSION_INDEX']
        return 'mock-session'

    def get_errors(self):
        return []

    def redirect_to(self, url):
        return url



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from django.conf import settings


def get_attribute(request, name):
    """
    Return the SAML attribute stored in session.samlUserdata.
    """
    value = request.session.get('samlUserdata', {}).get(name)

    if value is not None:
        if name not in ['affiliations', 'scopedAffiliations', 'isMemberOf']:
            return value[0] if len(value) else None

    return value


def get_user(request):
    """
    Return the user login stored in session.samlUserdata, identified in
    settings by either 'uwnetid' (default) or 'eppn'.
    """
    user_attr = getattr(settings, 'SAML_USER_ATTRIBUTE', 'uwnetid')
    return get_attribute(request, user_attr)


def is_member_of_group(request, group_id):
    """
    Utility function that checks whether the user is a member of the group
    identified by the passed group_id.
    """
    groups = get_attribute(request, 'isMemberOf') or []
    return group_id in groups



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


MOCK_SAML_ATTRIBUTES = {
    'urn:oid:0.9.2342.19200300.100.1.1': ['javerage'],
    'urn:oid:1.3.6.1.4.1.5923.1.1.1.1': ['student'],
    'urn:oid:1.3.6.1.4.1.5923.1.1.1.6': ['javerage@washington.edu'],
    'urn:oid:1.3.6.1.4.1.5923.1.1.1.9': ['student@washington.edu'],
    'urn:oid:1.3.6.1.4.1.5923.1.5.1.1': [
        'urn:mace:washington.edu:groups:u_test_group',
        'urn:mace:washington.edu:groups:u_test2_group'],
}


MOCK_SESSION_ATTRIBUTES = {
    'uwnetid': ['javerage'],
    'affiliations': ['student'],
    'eppn': ['javerage@washington.edu'],
    'scopedAffiliations': ['student@washington.edu'],
    'isMemberOf': ['u_test_group', 'u_test2_group'],
}

UW_SAML_PERMISSIONS = {
    'perm1': 'u_test_group',
    'perm2': 'u_test_another_group',
}

DJANGO_LOGIN_MOCK_SAML = {
    'NAME_ID': 'test-mock-nameid',
    'SESSION_INDEX': 'test-mock-session',
    'SAML_USERS': [
        {
            "username": "test_username",
            "password": "test_password",
            "email": "test_email",
            "MOCK_ATTRIBUTES": {
                'uwnetid': ["test_username"],
                'affiliations': ['student', 'member'],
                'eppn': ['javerage@washington.edu'],
                'scopedAffiliations': [
                    'student@washington.edu',
                    'member@washington.edu'
                ],
                'isMemberOf': [
                    UW_SAML_PERMISSIONS['perm1'],
                    UW_SAML_PERMISSIONS['perm2']
                ],
            }
        }
    ]
}



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from django.contrib.auth.models import User
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.contrib.auth.views import LoginView as DjangoLoginView
from django.http import HttpResponseRedirect
from django.test import RequestFactory
from django.urls import reverse_lazy
from django.utils.decorators import method_decorator
from django.views.generic.base import View, TemplateView
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_exempt
from uw_saml.auth import DjangoSAML


class UWSAMLView(TemplateView):
    template_name = 'uw_saml/sso_error.html'


@method_decorator(never_cache, name='dispatch')
class LoginView(UWSAMLView):
    def get(self, request, *args, **kwargs):
        return_url = request.GET.get(REDIRECT_FIELD_NAME)
        try:
            auth = DjangoSAML(request)
            return HttpResponseRedirect(auth.login(return_to=return_url))
        except KeyError as ex:
            context = {'errors': ['Missing: {}'.format(ex)]}
            return self.render_to_response(context, status=400)


@method_decorator(never_cache, name='dispatch')
class LogoutView(UWSAMLView):
    def get(self, request, *args, **kwargs):
        try:
            auth = DjangoSAML(request)
            return HttpResponseRedirect(auth.logout())
        except KeyError as ex:
            context = {'error_msg': 'Logout Failed',
                       'errors': ['Missing: {}'.format(ex)]}
            return self.render_to_response(context, status=400)


@method_decorator(csrf_exempt, name='dispatch')
class SSOView(UWSAMLView):
    http_method_names = ['post']

    def post(self, request, *args, **kwargs):
        try:
            auth = DjangoSAML(request)
        except KeyError as ex:
            context = {'errors': ['Missing: {}'.format(ex)]}
            return self.render_to_response(context, status=400)

        try:
            auth.process_response()
        except Exception as ex:
            context = {'error_msg': str(ex),
                       'errors': auth.get_errors()}
            return self.render_to_response(context, status=400)

        errors = auth.get_errors()
        if len(errors):
            return self.render_to_response({'errors': errors}, status=500)

        return_url = request.POST.get('RelayState')
        return HttpResponseRedirect(auth.redirect_to(return_url))


@method_decorator(csrf_exempt, name='dispatch')
class MockSSOLoginView(DjangoLoginView):
    """
    Overrides LoginView.form_valid() to insert a mocked SSO Login.
    """
    template_name = 'uw_saml/mock/login.html'

    def form_valid(self, form):
        response = super().form_valid(form)

        manufactured_request = RequestFactory().post(
            reverse_lazy('saml_sso'),
            data={'RelayState': self.get_success_url()}
        )
        manufactured_request.user = (
            User.objects.get(pk=self.request.session['_auth_user_id']))
        manufactured_request.session = self.request.session

        sso_view = SSOView.as_view()
        return sso_view(manufactured_request)



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from django.conf import settings
from django.urls import reverse, reverse_lazy, clear_url_caches
from django.core.exceptions import ImproperlyConfigured, PermissionDenied,\
    ImproperlyConfigured
from django.contrib.auth.models import AnonymousUser, User
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import TestCase, RequestFactory, override_settings
from uw_saml.auth import DjangoSAML, OneLogin_Saml2_Auth, Mock_Saml2_Auth,\
    Django_Login_Mock_Saml2_Auth
from uw_saml.tests import MOCK_SAML_ATTRIBUTES, MOCK_SESSION_ATTRIBUTES,\
    UW_SAML_PERMISSIONS, DJANGO_LOGIN_MOCK_SAML
import mock
import sys
from importlib import reload


@override_settings(MOCK_SAML_ATTRIBUTES=MOCK_SAML_ATTRIBUTES)
class MockAuthTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().post(
            reverse('saml_sso'), HTTP_HOST='example.uw.edu')
        self.request.user = None
        SessionMiddleware().process_request(self.request)
        self.request.session.save()

    def test_implementation(self):
        auth = DjangoSAML(self.request)
        self.assertIsInstance(auth._implementation, Mock_Saml2_Auth)
        self.assertEquals(auth.get_nameid(), 'mock-nameid')
        self.assertEquals(auth.get_session_index(), 'mock-session-index')

    def test_get_attributes(self):
        auth = DjangoSAML(self.request)
        self.assertEquals(auth.get_attributes(), {
            'affiliations': ['student'],
            'eppn': ['javerage@washington.edu'], 'uwnetid': ['javerage'],
            'scopedAffiliations': ['student@washington.edu'],
            'isMemberOf': ['u_test_group', 'u_test2_group']})

        with self.settings(MOCK_SAML_ATTRIBUTES=MOCK_SESSION_ATTRIBUTES):
            auth = DjangoSAML(self.request)
            self.assertEquals(auth.get_attributes(), {
                'affiliations': ['student'],
                'eppn': ['javerage@washington.edu'], 'uwnetid': ['javerage'],
                'scopedAffiliations': ['student@washington.edu'],
                'isMemberOf': ['u_test_group', 'u_test2_group']})

    def test_login(self):
        auth = DjangoSAML(self.request)
        url = auth.login(return_to='/test')
        self.assertEquals(url, '/test')
        self.assertEquals(self.request.user.is_authenticated, True)
        self.assertEquals(self.request.user.username, 'javerage')

        # Missing return_to arg
        auth = DjangoSAML(self.request)
        self.assertEquals(auth.login(), '')

    def test_logout(self):
        auth = DjangoSAML(self.request)
        url = auth.logout(return_to='/test')
        self.assertEquals(url, '/test')
        self.assertEquals(self.request.user.is_authenticated, False)
        self.assertEquals(self.request.user.username, '')

        # Missing return_to arg
        auth = DjangoSAML(self.request)
        self.assertEquals(auth.logout(), '')

    def test_nonexistent_method(self):
        auth = DjangoSAML(self.request)
        self.assertRaises(AttributeError, auth.fake_method)


@override_settings(
    UW_SAML_PERMISSIONS=UW_SAML_PERMISSIONS,
    DJANGO_LOGIN_MOCK_SAML=DJANGO_LOGIN_MOCK_SAML,
    AUTHENTICATION_BACKENDS=('django.contrib.auth.backends.ModelBackend',)
)
class DjangoLoginAuthTest(TestCase):
    def setUp(self):
        clear_url_caches()
        reload(sys.modules['uw_saml.urls'])
        reload(sys.modules[settings.ROOT_URLCONF])
        self.request = RequestFactory().post(
            reverse('saml_sso'), data={'SAMLResponse': ''},
            HTTP_HOST='idp.uw.edu')
        SessionMiddleware().process_request(self.request)
        self.request.session.save()

    def test_implementation(self):
        auth = DjangoSAML(self.request)
        self.assertIsInstance(
            auth._implementation,
            Django_Login_Mock_Saml2_Auth
        )

    def test_login(self):
        auth = DjangoSAML(self.request)
        self.assertEqual(
            auth.login(return_to='return_to_url'),
            "{}?next={}".format(
                reverse_lazy('login_django'),
                'return_to_url'
            )
        )

    def test_logout(self):
        auth = DjangoSAML(self.request)
        self.assertEqual(
            auth.logout(return_to='return_to_url'),
            "{}".format('return_to_url')
        )

    def test_bad_process_response(self):
        self.request.user = AnonymousUser()
        auth = DjangoSAML(self.request)
        with self.assertRaises(PermissionDenied):
            auth.process_response()

    def test_get_attributes(self):
        auth = DjangoSAML(self.request)
        auth._implementation.username = 'test_username'
        self.assertEquals(
            auth.get_attributes(),
            settings.DJANGO_LOGIN_MOCK_SAML['SAML_USERS'][0]['MOCK_ATTRIBUTES']
        )

    def test_bad_get_attributes(self):
        auth = DjangoSAML(self.request)
        auth._implementation.username = 'test_not_username'
        with self.assertRaises(ImproperlyConfigured):
            auth.get_attributes()

    def test_get_nameid(self):
        auth = DjangoSAML(self.request)
        self.assertEquals(
            auth.get_nameid(),
            'test-mock-nameid'
        )
        del settings.DJANGO_LOGIN_MOCK_SAML['NAME_ID']
        self.assertEquals(
            auth.get_nameid(),
            'mock-nameid'
        )

    def test_get_session_index(self):
        auth = DjangoSAML(self.request)
        self.assertEquals(
            auth.get_session_index(),
            'test-mock-session'
        )
        del settings.DJANGO_LOGIN_MOCK_SAML['SESSION_INDEX']
        self.assertEquals(
            auth.get_session_index(),
            'mock-session'
        )


class LiveAuthTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().post(
            reverse('saml_sso'), data={'SAMLResponse': ''},
            HTTP_HOST='idp.uw.edu')
        SessionMiddleware().process_request(self.request)
        self.request.session.save()

    @override_settings()
    def test_missing_implementation(self):
        del settings.UW_SAML
        self.assertRaises(ImproperlyConfigured, DjangoSAML, self.request)

    def test_implementation(self):
        auth = DjangoSAML(self.request)
        self.assertIsInstance(auth._implementation, OneLogin_Saml2_Auth)

    @mock.patch.object(OneLogin_Saml2_Auth, 'login')
    def test_login(self, mock_login):
        auth = DjangoSAML(self.request)
        url = auth.login(return_to='/test')
        mock_login.assert_called_with(force_authn=False, return_to='/test')

        with self.settings(SAML_FORCE_AUTHN=True):
            auth = DjangoSAML(self.request)
            url = auth.login(return_to='/test')
            mock_login.assert_called_with(force_authn=True, return_to='/test')

    @mock.patch.object(OneLogin_Saml2_Auth, 'get_attributes')
    def test_get_attributes(self, mock_attributes):
        mock_attributes.return_value = MOCK_SAML_ATTRIBUTES

        auth = DjangoSAML(self.request)
        self.assertEquals(auth.get_attributes(), {
            'affiliations': ['student'],
            'eppn': ['javerage@washington.edu'], 'uwnetid': ['javerage'],
            'scopedAffiliations': ['student@washington.edu'],
            'isMemberOf': ['u_test_group', 'u_test2_group']})



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


import mock
import json
from django.conf import settings
from django.urls import reverse
from django.contrib.auth.models import User
from django.contrib.auth.middleware import AuthenticationMiddleware
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import TestCase, RequestFactory, override_settings
from uw_saml.views import LoginView, LogoutView, SSOView, MockSSOLoginView
from uw_saml.auth import OneLogin_Saml2_Auth, Django_Login_Mock_Saml2_Auth
from uw_saml.tests import (
    MOCK_SAML_ATTRIBUTES, UW_SAML_PERMISSIONS, DJANGO_LOGIN_MOCK_SAML)

CACHE_CONTROL = 'max-age=0, no-cache, no-store, must-revalidate'


class LoginViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get(
            reverse('saml_login'), HTTP_HOST='example.uw.edu')
        SessionMiddleware().process_request(self.request)
        self.request.session.save()

    def test_login(self):
        view_instance = LoginView.as_view()
        response = view_instance(self.request)
        self.assertEquals(response.status_code, 302)
        self.assertIn(settings.UW_SAML['idp']['singleSignOnService']['url'],
                      response.url)
        self.assertIn(CACHE_CONTROL, response.get('Cache-Control'))

    def test_missing_request_data(self):
        # Missing HTTP_HOST
        request = RequestFactory().get(reverse('saml_login'))
        SessionMiddleware().process_request(request)
        self.request.session.save()

        view_instance = LoginView.as_view()
        response = view_instance(request).render()
        self.assertContains(
            response, 'SSO Error: Login Failed', status_code=400)
        self.assertContains(
            response, 'Missing: &#39;HTTP_HOST&#39;', status_code=400,
            html=True)


class LogoutViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get(
            reverse('saml_logout'), HTTP_HOST='example.uw.edu')
        SessionMiddleware().process_request(self.request)
        self.request.session['samlNameId'] = ''
        self.request.session['samlSessionIndex'] = ''
        self.request.session.save()

    def test_logout(self):
        view_instance = LogoutView.as_view()
        response = view_instance(self.request)
        self.assertEquals(response.status_code, 302)
        self.assertIn(settings.UW_SAML['idp']['singleLogoutService']['url'],
                      response.url)
        self.assertIn(CACHE_CONTROL, response.get('Cache-Control'))

    def test_missing_request_data(self):
        # Missing HTTP_HOST
        request = RequestFactory().get(reverse('saml_logout'))
        SessionMiddleware().process_request(request)
        self.request.session.save()

        view_instance = LogoutView.as_view()
        response = view_instance(request).render()
        self.assertContains(
            response, 'SSO Error: Logout Failed', status_code=400)
        self.assertContains(
            response, 'Missing: &#39;HTTP_HOST&#39;', status_code=400,
            html=True)


class SSOViewTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().post(
            reverse('saml_sso'),
            data={'RelayState': '/private'},
            HTTP_HOST='example.uw.edu')
        SessionMiddleware().process_request(self.request)
        self.request.session.save()

    @mock.patch.object(OneLogin_Saml2_Auth, 'get_attributes')
    @mock.patch.object(OneLogin_Saml2_Auth, 'process_response')
    def test_sso(self, mock_process_response, mock_get_attributes):
        mock_get_attributes.return_value = MOCK_SAML_ATTRIBUTES

        view_instance = SSOView.as_view()
        response = view_instance(self.request)
        self.assertEquals(response.status_code, 302)
        self.assertEquals(response.url, 'http://example.uw.edu/private')


class SSOViewErrorTest(TestCase):
    def test_missing_request_data(self):
        # Missing HTTP_HOST
        request = RequestFactory().post(reverse('saml_sso'))
        SessionMiddleware().process_request(request)
        request.session.save()

        view_instance = SSOView.as_view()
        response = view_instance(request).render()
        self.assertContains(
            response, 'SSO Error: Login Failed', status_code=400)
        self.assertContains(
            response, 'Missing: &#39;HTTP_HOST&#39;', status_code=400,
            html=True)

    def test_missing_post_data(self):
        request = RequestFactory().post(
            reverse('saml_sso'), HTTP_HOST='example.uw.edu')
        SessionMiddleware().process_request(request)
        request.session.save()

        view_instance = SSOView.as_view()
        response = view_instance(request)
        self.assertContains(response, 'SSO Error:', status_code=400)

    def test_invalid_saml_response(self):
        request = RequestFactory().post(
            reverse('saml_sso'), data={'SAMLResponse': ''},
            HTTP_HOST='idp.uw.edu')
        SessionMiddleware().process_request(request)
        request.session.save()

        view_instance = SSOView.as_view()
        response = view_instance(request)
        self.assertContains(response, 'SSO Error:', status_code=400)

    def test_invalid_http_method(self):
        request = RequestFactory().get(
            reverse('saml_sso'), HTTP_HOST='idp.uw.edu')
        SessionMiddleware().process_request(request)
        request.session.save()

        view_instance = SSOView.as_view()
        response = view_instance(request)
        self.assertEquals(response.status_code, 405)


@override_settings(
    UW_SAML_PERMISSIONS=UW_SAML_PERMISSIONS,
    DJANGO_LOGIN_MOCK_SAML=DJANGO_LOGIN_MOCK_SAML,
    AUTHENTICATION_BACKENDS=('django.contrib.auth.backends.ModelBackend',)
)
class DjangoLoginViewTest(TestCase):
    def setUp(self):
        self.request_factory = RequestFactory()

    def tearDown(self):
        User.objects.all().delete()

    def test_login_valid(self):
        req = self.request_factory.post(
            'login_django',
            data={'username': 'test_username', 'password': 'test_password'},
        )
        SessionMiddleware().process_request(req)
        req.session.save()
        req._dont_enforce_csrf_checks = True

        # Initalized so the users are loaded
        Django_Login_Mock_Saml2_Auth(req)
        resp = MockSSOLoginView.as_view()(req)
        self.assertEqual(resp.status_code, 302)

    def test_login_invalid(self):
        req = self.request_factory.post(
            'login_django',
            data={
                'username': 'test_username',
                'password': 'test_wrong_password'
            },
        )
        SessionMiddleware().process_request(req)
        req.session.save()
        AuthenticationMiddleware().process_request(req)
        req._dont_enforce_csrf_checks = True
        # req.user = AnonymousUser()

        # Initalized so the users are loaded
        Django_Login_Mock_Saml2_Auth(req)
        resp = MockSSOLoginView.as_view()(req)
        self.assertEqual(resp.status_code, 200)



# Copyright 2022 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from django.conf import settings
from django.conf.urls import re_path
from uw_saml.views import LoginView, LogoutView, SSOView, MockSSOLoginView


urlpatterns = [
    re_path(r'login$', LoginView.as_view(), name='saml_login'),
    re_path(r'logout$', LogoutView.as_view(), name='saml_logout'),
    re_path(r'sso$', SSOView.as_view(), name='saml_sso'),
]

if (hasattr(settings, 'DJANGO_LOGIN_MOCK_SAML')):
    urlpatterns.append(re_path(
        r'^login_django$', MockSSOLoginView.as_view(), name='login_django')
    )
